# import mwparserfromhell
import re
import ujson as json
from loguru import logger
import os
import inspect
from mako.template import Template
from mako.runtime import Context

def get_keys(dict,value)->list:
	'''[dict]用于获取键内容相同的所有值'''
	l = [k for k,v in dict.items() if v==value]
	if l :
		return l
	else:
		logger.warning(f'请检查字典{dict}中是否存在值为{value}的键!')
		return []

def get_all_links(content:str)->list:
    page_list = re.findall(pattern='(?<=\[\[)(.+?)(?=(\|.*?)|(\]\]))', string=content)
    return [i[0] for i in page_list]

# def get_page_links_from_pagelist_txt(content=os.path.dirname(__file__))->list:
def get_page_links_from_pagelist_txt(content=os.path.dirname(inspect.stack()[1].filename))->list:
	classes_path = os.path.expanduser(f'{content}/pagelist.txt')
	with open(classes_path,'r',encoding = 'UTF-8') as f:
		pagelist = f.readlines()
	pagelist = [c.rstrip() for c in pagelist]
	return pagelist

def template_from_file(t_name:str,**kwargs)->str:
    template = Template(filename=os.path.expanduser(f'{os.path.dirname(inspect.stack()[1].filename)}/templates/{t_name}'),strict_undefined=True)  
    return template.render(**kwargs).strip()


import httpx
import ujson as json
from loguru import logger
import os
from .prototype import WikiSectionDict
from typing import Union


class Bot:
    '''(https://www.mediawiki.org/wiki/API:Main_page/zh)[Mediawiki文档]
    Bot根程序。
    :param sitename(`str`) : 站点名称
    :param index(`str`) : index.php目录
    :param api(`str`) : index.php路径
    :param index(`str`) : api.php路径
    :param username(`str`) : 用户名
    :param password(`str`) : 用户密码'''

    # 成员变量
    def __init__(self,sitename:str,api:str,index:str,username:str,password:str):
        '''同步的构造函数'''
        self.sitename = sitename
        self.api = api
        self.index = index
        self.username = username
        self.password = password
        timeout = httpx.Timeout(10.0, connect=60.0, read=60.0, write=60.0, pool=60.0)
        self.client = httpx.AsyncClient(verify=False,timeout=timeout)
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.62 Safari/537.36'}

    async def __aexit__(self):
        '''异步析构函数'''
        await self.client.aclose()
        logger.info("This task has been completed!")

    async def fetch_token(self, type:str)->str:
        '''根据不同的type类型返回对应的token
        :param type(`str`) : token的类型
        :return: (`str`)<token>'''
        PARAMS = {
            'action': "query",
            'meta': "tokens",
            'type': type,
            'format': "json"
        }
        token = await self.client.post(url=self.api, data=PARAMS)
        token.raise_for_status()
        token = token.json()
        location = type + "token"
        return token['query']['tokens'][location]

    async def login(self):
        '''登录'''
        data = {
            'action': "login",
            'lgname': self.username,
            'lgpassword': self.password,
            'lgtoken': await self.fetch_token(type="login"),
            'format': "json"
        }
        act = await self.client.post(url=self.api, data=data,headers=self.headers)
        act = act.json()
        if act['login']['result'] == "Success":
            logger.info(f'Welcome to {self.sitename}, {act["login"]["lgusername"]}!')
            print(act)

    async def get_data(self, title:str):
        PARAMS = {
            "action": "query",
            "prop": "revisions",
            "titles": title,
            "rvslots": "*",
            "rvprop": "content",
            "formatversion": 2,
            "format": "json"
        }
        text = await self.client.post(url=self.api, data=PARAMS, headers=self.headers)
        text = text.json()
        text = text["query"]["pages"][0]
        # logger.info(f'Get info of [[{text["title"]}]] successfully.')
        return text

    async def get_page_text(self,title:str,section:Union[str,int]='')->str:
        '''获取页面中的文本'''
        # PARAMS = {
        #     "title=": title,
        #     "action": "raw",
        #     "section": section
        # }
        # act = self.S.post(url=self.index, data=PARAMS, headers=self.headers)
        act = await self.client.post(url=f"{self.index}?action=raw&title={title}&section={str(section)}", headers=self.headers)
        if act.status_code == 404:
            logger.warning(f"请检查get_page_text传入的页面是否在{self.sitename}存在。")
            return None
        else:
            return str(act.text)

    async def edit_page(self,title:str,text:str,summary:str="", **kwargs):
        '''编辑一个页面。常用参数：title,text,summary.'''
        PARAMS = {
            "action": "edit",
            "minor": True,
            "bot": True,
            "format": "json",
            "title": title,
            "text": text,
            "summary":summary
        }
        for key, value in kwargs.items():
            key = str(key)
            value = str(value)
            PARAMS[key] = value
        PARAMS["token"] = await self.fetch_token(type="csrf")
        PARAMS["summary"] += " //Edit by Bot."
        act = await self.client.post(url=self.api, data=PARAMS, headers=self.headers)
        act = act.json()
        if ('edit'['result'] in act) & (act['edit'][result]== "Success"):
            logger.info(f'Edit [[{PARAMS["title"]}]] successfully.')
        else:
            logger.debug(act)

    async def create_page(self,title:str,text:str,summary:str="")->bool:
        '''创建页面'''
        deal = await self.get_data(title=title)
        if "missing" in deal:
            await self.edit_page(title=title,text=text,summary=summary)
            return False
        else:
            logger.info(f"Skip Create [[{title}]].")
            return True

    async def upload_local(self,local_name,local_path,web_name,text="", **kwargs):
        '''从本地上传一个文件.'''
        PARAMS = {
            "action": "upload",
            "filename": web_name,
            "format": "json",
            "token": self.fetch_token(type="csrf"),
            "ignorewarnings": True,
            "watchlist": "nochange",
            "async": True
        }
        for key, value in kwargs.items():
            key = str(key)
            value = str(value)
            PARAMS[key] = value
        FILE = {'file': (local_name, open(local_path, 'rb'), 'multipart/form-data')}
        act = await self.client.post(url=self.api, data=PARAMS, headers=self.headers, files=FILE)
        act = act.json()
        if act["upload"]["result"] == "Success":
            logger.info(f'Upload {local_name}=>[[File:{web_name}]] successfully.')
        else:
            logger.debug(act)

    async def purge(self,title:str,**kwargs):
        '''刷新页面'''
        PARAMS = {
            "action": "purge",
            "titles": title,
            "format": "json"
        }
        for key, value in kwargs.items():
            key = str(key)
            value = str(value)
            PARAMS[key] = value
        act = await self.client.post(url=self.api, data=PARAMS,headers=self.headers)
        act = act.json()
        logger.info(f"Purge [[{title}]] Successfully.")

    async def parse(self, title, **kwargs):
        '''解析'''
        PARAMS = {
            "format": "json",
            "page": title,
            "action": "parse"
        }
        for key, value in kwargs.items():
            key = str(key)
            value = str(value)
            PARAMS[key] = value
        act = await self.client.post(url=self.api, data=PARAMS, headers=self.headers)
        return act.json()

    async def get_sections(self, title:str)->WikiSectionDict:
        result = await self.parse(title=title, prop='sections')
        result = result['parse']['sections']
        result_list = []
        for i in result:
            result_list.append(i['line'])
        if result_list:
            return WikiSectionDict(result_list)
        else:
            return False
            logger.info(f'Page [[{title}]] has no section!')

    async def deal_flow(self,title,cotmoderationState,cotreason="标记"):
        PARAMS = {
            "action": "flow",
            "page": str(title),
            "submodule":"lock-topic", 
            "cotmoderationState":cotmoderationState,
            "cotreason":cotreason,
            "format": "json",
            "token":self.fetch_token(type="csrf")
        }
        act = await self.client.post(url=self.api, data=PARAMS, headers=self.headers).json()
        logger.info(f"{cotmoderationState} the flow {title} successfully.({cotreason})")

    async def reply_flow(self,title,content):
        PARAMS = {
            "action": "flow",
            "submodule":"reply",
            "page":title,
            "repreplyTo": str(title),
            "repcontent":str(content),
            "repformat":"wikitext",
            "format": "json",
            "token":self.fetch_token(type="csrf")
        }
        act = await self.client.post(url=self.api, data=PARAMS, headers=self.headers).json()
        logger.info(f"Reply the flow {title} successfully.")

    def rc(self,namespace):
        ...
    async def search(self, txt:str, namespace:str="0", sroffset:str="0", **kwargs):
        PARAMS = {
            "action":"query",
            "list":"search",
            "srsearch":txt,
            "srlimit":"max",
            "utf8":"",
            "srnamespace":namespace,
            "srwhat":"text",
            "sroffset":sroffset,
            "format": "json",
        }
        for key, value in kwargs.items():
            key = str(key)
            value = str(value)
            PARAMS[key] = value
        act = await self.client.post(url=self.api, data=PARAMS, headers=self.headers)
        act = act.json()
        rl = []
        if act["query"]["search"] != False: 
            print(len(act["query"]["search"]),sroffset)
            for i in act["query"]["search"]:
                rl.append(i["title"])    
        if "continue" in act:
            temp = await self.search(txt=txt,namespace=namespace,sroffset=(int(sroffset)+1),**kwargs)
            for i in temp:rl.append(i)
        return rl

    async def ask(self,query:str,api_version:int=2):
        PARAMS = {
            "action":"ask",
            "query":query,
            "api_version":api_version,
            "format": "json",
        }
        act = await self.client.post(url=self.api, data=PARAMS, headers=self.headers)
        act = act.json()
        return act["query"]

    async def protect(self,title,protections,expiry:str="infinite",reason:str="",**kwargs):
        PARAMS = {
            "action": "protect",
            "format": "json",
            "title": title,
            "protections":protections,
            "expiry":expiry,
            "reason":reason
        }
        for key, value in kwargs.items():
            key = str(key)
            value = str(value)
            PARAMS[key] = value
        PARAMS["token"] = await self.fetch_token(type="csrf")
        PARAMS["reason"] += " //Protect by Bot."
        act = await self.client.post(url=self.api, data=PARAMS, headers=self.headers)
        act = act.json()
        return act["protect"]


# from bot import Bot
# import asyncio
# async def main():
#     bot = Bot(
#             sitename="Guki",
#             api="",
#             index="",
#             username="GuBot",
#             password="MainBot@")
#     await bot.login()
#     r = await bot.get_section(page_name="用户:咕咕mur")
#     print(r)
#     await bot.close()

# if __name__ == "__main__":
#     asyncio.run(main())
from mwbot import arktool
arktool.GameDataPosition = "/home"
arktool.read_ark_cn_file("excel/item_table.json")


from .bot import Bot
import nonebot
import asyncio
bot = Bot(
			sitename = "PRTS",
			api = "https://prts.wiki/api.php", 
			index = "https://prts.wiki/index.php",
			username="GuBot",
			password="MainBot@av89f6mgb7ns4m2ul8gaa8sdhq30altt")

if __name__ == "__main__":
    asyncio.run(main())


'''mwbot的工具集'''
import re, os
import ujson as json
import datetime
from loguru import logger
from jinja2 import Environment, FileSystemLoader

__all__ = ["get_all_links","get_page_links_from_pagelist_txt","render_template"]
def get_all_links(content:str)->list:
    '''从一段文字中获取其中所有的链接
    .. code-block:: python
        linklist = utils.get_all_links(content)
    :param content: 包含链接的字符串
    :returns: [list]
    '''
    page_list = re.findall(pattern='(?<=\[\[)(.+?)(?=(\|.*?)|(\]\]))', string=content)
    return [i[0] for i in page_list]

def get_page_links_from_pagelist_txt(folder=os.path.dirname(os.path.abspath(__file__)))->list:
    '''从bot文件同目录下的pagelist.txt读取每行对应的页面名并返回列表。
    
    .. code-block:: python
        pagelist = utils.get_page_links_from_pagelist_txt()
    :param folder: [可选项]pagelist.txt所在的文件夹
    :returns: list'''
    classes_path = os.path.expanduser(f'{folder}/pagelist.txt')
    with open(f"{folder}/pagelist.txt",'r',encoding = 'UTF-8') as f:
        pagelist = f.readlines()
    pagelist = [c.rstrip() for c in pagelist]
    return pagelist

def render_template(T_NAME: str,**kwargs) -> str:
    '''从 bot程序 目录下的 /templates 目录中选择Jinja2模板并渲染
    .. code-block:: python
        utils.render_template(T_NAME="test.jinja",name="Test")
    :param T_NAME: ./templates 目录中的模板文件名
    :param **kwargs: 对应模板要求的参数
    :return: 渲染后的结果'''
    T_ENV = Environment(loader=FileSystemLoader(f'{os.path.dirname(os.path.abspath(__file__))}/templates'))
    return T_ENV.get_template(T_NAME).render(**kwargs).strip()


'''定义一些功能特化的基类'''
from abc import ABC
import mwparserfromhell
# import pendulum
# import regex as re

class WikiSectionList(ABC):
    '''用于Bot.get_section()方法的字典类
    覆写了dict.index()方法，输出可直接用于edit(section)的值'''
    def __init__(self, wl:list):
        self.wd = wl
    def index(num:str)->int:
        return self.wl.index(num)+1
    def __str__(self):
        return str(self.wl)
    
class gen_wikitext:
    '''生成wikitext'''
    def __init__(self):
        self.txt:str = """"""
    def add_title(self,po:int,title:str):
        self.txt += "="*po + title + "="*po + "\n"
    def add_text(self,text:str):
        self.txt += (text + "\n")
    def add_cat(self,cat:str):
        self.txt += f"[[分类:{cat}]]\n"
        
class gen_tem:
    '''生成wikitext-template'''
    def __init__(self,title:str):
        self.txt = "{{"+title
        self.enter_int = 0
    def add_par(self,name:str = "", txt:str = "", enter:bool=True):
        '''添加一行参数。'''
        if bool == True:
            self.txt += "\n"
            self.enter_int += 1
        if name:
            self.txt+=f"|{name}={txt}"
        else:
            self.txt += f"|{txt}"
    def result(self)->str:
        '''输出结果'''
        if self.enter_int != 0:
            self.txt += "\n"
        self.txt += "}}"
        return self.txt


'''存储与PRTS编辑相关的方法'''
import mwparserfromhell
import re
import requests
import ujson as json
from loguru import logger
import os
from pathlib import Path

gamedataPosition = "/home/bot/ArknightsGameData/zh_CN/gamedata"

def get_item_name(id:str)->str:
    """输入：`id`
    输出：对应的道具名称"""
    return read_ark_cn_file("excel/item_table.json")['items'][id]['name']

def deal_item_info(type:str,id:str,droptype:int)->str:
    link = read_ark_cn_file("excel/item_table.json")["items"][id]
    name = link["name"]
    if droptype == 8:
        kind = "三星获得"
    return f"{name}:{kind}"

def catch_item_template(id:str,count:int)->str:
    return "{{材料消耗|"+str(read_ark_cn_file("excel/item_table.json")["items"][id]["name"])+"|"+str(count)+"}}"

def read_ark_cn_file(filename):
    return json.loads(Path(f"{gamedataPosition}/{filename}").read_text())

def trap_set(page_name:str, trap_list:list):
    ...

def get_stage_id(content):
    wikicode = mwparserfromhell.parse(content)
    templates = wikicode.filter_templates()
    for i in templates:
        if i.name.matches('普通关卡信息') or i.name.matches('剿灭关卡信息'):
            temp = i
            break
    stage_id = temp.get('关卡id').value
    return stage_id.strip()

def get_stage_info(content):
    stage_id = get_stage_id(content=content)
    stage_id_location = read_ark_cn_file("excel/stage_table.json")['stages'][stage_id]['levelId']
    if stage_id_location == None:
        return None
    else:
        stage_id_location = stage_id_location.lower()
        return read_ark_cn_file('gamedata/levels/{stage_id_location}.json')

def get_char_name(id:str)->str:
    return read_ark_cn_file("character_table")[id]["name"]